# -*- coding: utf-8 -*-

from subprocess import run
from io import BytesIO

from requests import get
from pandas import read_csv 
from .exceptions import InvalidAccessionError, ProgramMissingError

class HiddenMarkovModel:
    """Class for a profile Hidden Markov Model."""

    def __init__(self, accession):
        self.accession = accession
        self.hmm = self.get_hmm(accession)
        self.logo = self.get_logo(self.hmm)
        self.logo_df = self.get_logo_df()


    def get_hmm(self, accession):
        url = f'https://pfam.xfam.org/family/{accession}/hmm'
        response = get(url=url)
        if not response.ok:
            raise InvalidAccessionError(accession)
        hmm = response.content
        return hmm

    
    def get_logo(self, hmm):
        args = "hmmlogo --no_indel /dev/stdin".split()
        try:
            proc = run(args, input=hmm, capture_output=True)
        except (FileNotFoundError):
            raise ProgramMissingError(args)
        logo = proc.stdout
        return logo

    
    def get_logo_df(self):
        logo = BytesIO(self.logo)
        df = self.read_logo(logo)
        return df

    
    def read_logo(self, logo):
        names = ['profile']+list('ACDEFGHIKLMNPQRSTVWY-')
        sep = '\s+\(.*|:?\s+'
        df = read_csv(logo, index_col=0, skiprows=2, header=None,
                      sep=sep, engine='python', names=names)
        df = df.dropna(axis='columns')
        return df
        




# -*- coding: utf-8 -*-

class InvalidAccessionError(Exception):
    """Exception raised when using an invalid Pfam accession number"""

    def __init__(self, accession):
        self.accession = accession
        self.message = f'No Hidden Markov Model found with accession number: {accession}.'
        super().__init__(self.message)

class ProgramMissingError(Exception):
    """Exception raised when a returncode different from zero is returned."""

    def __init__(self, args):
        self.args = args
        self.command = ' '.join(args)
        self.message = f"""The following command failed to run on your system:

        {self.command}

        Make sure that {args[0]} is installed."""
        super().__init__(self.message)



# -*- coding: utf-8 -*-

from .plot import LogoPlot

def logofig(accession, start=None, end=None):
    hmmlogoplot = LogoPlot(accession, start=start, end=end)
    hmmlogofig = hmmlogoplot.get_logofig()
    return hmmlogofig






# -*- coding: utf-8 -*-

from json import load
from base64 import b64encode
from io import BytesIO, StringIO
from os.path import dirname, abspath, join

import matplotlib.pyplot as plt
import numpy as np

from matplotlib.path import Path
from matplotlib.patches import PathPatch
from .hmm import HiddenMarkovModel

here = dirname(abspath(__file__))
alph_path = join(here,'alph.json')
cmap_path = join(here,'cmap.json')

class LogoPlot:

    def __init__(self, accession, start=None, end=None, alph=alph_path, cmap=cmap_path):
        self.accession = accession
        self.start = start
        self.end = end
        self.hmm = HiddenMarkovModel(accession)
        self.logo = self.hmm.logo_df
        self.partial_logo = self.logo.loc[start:end]
        self.alph = self.read_json(alph)
        self.cmap = self.read_json(cmap)
        self.get_logoplot()

    def get_logoplot(self):
        self.fig, self.ax = plt.subplots(figsize=(len(self.partial_logo)/4,2), tight_layout=True)
        self.ax = self.plot_logo()

    def plot_logo(self):
        y_max = 0
        for x_offset, (_, series) in enumerate(self.partial_logo.iterrows()):
            self.plot_column(series, x_offset)
            y_max = max(y_max, series.sum())
        self.ax.axis([0, len(self.partial_logo), 0, y_max])
        self.ax.set_xticks(np.arange(len(self.partial_logo)+0.5))
        self.ax.set_xticklabels(self.partial_logo.index)

    def plot_column(self, series, x_offset):
        y_offset = 0
        series = series.sort_values()
        for let, y_scale in series.iteritems():
            patch = self.plot_letter(let, scale=(1, y_scale), offset=(x_offset, y_offset))
            self.ax.add_patch(patch)
            y_offset += y_scale

    def plot_letter(self, let, scale=(1, 1), offset=(0,0)):
        path = Path(*self.alph[let])
        path.vertices *= list(scale)
        path.vertices += list(offset)
        patch = PathPatch(path, color=self.cmap[let])
        return patch

    def read_json(self, path):
        with open(path, 'r') as f:
            obj = load(f)
        return obj

    def get_logofig(self, format='png'):
        if format == 'svg':
            logofig = StringIO()
        elif format == 'png':
            logofig = BytesIO()
        self.fig.savefig(logofig, format=format)
        logofig.seek(0)
        if format == 'svg':
            logofig = ''.join(logofig.readlines()[4:])
        elif format == 'png':
            logofig = b64encode(logofig.read()).decode()
            logofig = f'<img alt="" src="data:image/png;base64,{logofig}">'
        return logofig
